#***************************************************************************************************************
# Name :          Delete Coplanar Edges 1.2
# Description :   deletes coplanar edges of a selection
# Author :        Jack Dolabany (jdolabany@TROarch.com). Based off of 
#                 [CASF]DelCopEdges by Carlos Antnio dos Santos Fal (carlosfale@sapo.pt)
# Usage :         Select entities and right click. Go to Delete Coplanar Edges (strict or loose) 
#                 and try to delete the edges.
# Date :          July.2oo5
# Type :          Tool
#***************************************************************************************************************
require 'Sketchup.rb'


class Copedges

  # level of selection
  @down_level = 2
  
  # validate the selection
  def Copedges::validate_selection
    sel = Sketchup.active_model.selection
    for ss in sel
      if ss.kind_of? Sketchup::Edge
        return true
      end
    end
    return false
  end #def Copedges
  
  
  
  #I use this function for testing purposes
  #just ignore it if you are trying to understand the code.
  def Copedges::prep_faces
    model = Sketchup.active_model
    model.start_operation "Prep Faces"
    sel = model.selection
    vector = Geom::Vector3d.new 0,0,1
    i=0
    erase_hidden_faces
    model.entities.length.times do
      #if sel[i].class == Sketchup::Face
        #if sel[i].normal == vector
         # sel[i].pushpull 1000
          #sel[i].pushpull -1000
        #end
      #end
      i += 1
    end
    model.commit_operation #Prep Faces
  end
  
  
  

  #***************************************************************************************************************
  #                                     ***erase_hidden_faces***
  #     A hidden face in a simple object is defined here as something that has three faces attached to each edge.
  # This function finds those faces and erases them.
  #***************************************************************************************************************
  
  def Copedges::erase_hidden_faces
    model = Sketchup.active_model
    model.start_operation "erase_hidden_faces"
    sel = model.selection
    ind = 0
    while ind < sel.length
      flag = 0
      if sel[ind].class == Sketchup::Face
        if sel[ind].edges[0].faces.length >= 3
          flag += 1
        end
        if sel[ind].edges[1].faces.length >= 3
          flag += 1
        end
        if sel[ind].edges[2].faces.length >= 3
          flag += 1
        end
        if sel[ind].edges[3] != nil #make sure this edge exists. It wont if its a triangle
          if sel[ind].edges[3].faces.length >= 3
            flag += 1
          end
        end
        if flag >= 3
          #puts "hidden face"
          sel[ind].erase! #all four edges have two attached planes
        end
      end #if thing.class == Sketchup::Face
      ind += 1
    end #while
    model.commit_operation #erase hidden faces
  end #def Copedges
  
  #***************************************************************************************************************
  #                                     ***Delete_Coplanar_Edges_Loose***
  #     This function checks selections of entities for coplanar edges and deletes them. The function
  # recognizes coplanar faces and two faces along one edge that have the same or opposite normal.
  # In 3D objects Edges seem to have not more than 4 connected faces, so that is the scope of the function
  # The function works by scrolling through every element in the selection. If an element is an edge,
  # all of the attached faces are compared to see if their normals are equal or opposite. 
  #     For the case with three joined faces, faces 0&1, 0&2, and 1&2 are compared. For the case with four 
  # joined faces, faces 0&1, 0&2, 0&3, 1&2, 1&3, and 2&3 are compared. Since the function isn't perfect, the
  # commented out flags can be used for testing.
  #***************************************************************************************************************
  
  def Copedges::delete_coplanar_edges_loose
  
    model = Sketchup.active_model
    model.start_operation "Delete Coplanar Edges Loose"
    
    sel = model.selection
    2.times do
    if sel.length < @down_level
      @down_level = sel.length
    end
    #flag = 0  
    ind = 0
      
    #Function to find and delete hidden faces.
    erase_hidden_faces
    erase_hidden_faces
    while ind < sel.length
      if sel[ind].class == Sketchup::Edge
        
        if sel[ind].faces.length == 0
          sel[ind].erase!
          ind = ind - @down_level
          flag = 1
          
        elsif sel[ind].faces.length == 2
          if (sel[ind].faces[0].normal.angle_between(sel[ind].faces[1].normal) == 0)
            sel[ind].erase!
            ind = ind - @down_level
            #flag = 1
          elsif (sel[ind].faces[0].normal.angle_between(sel[ind].faces[1].normal.reverse!) == 0)
              sel[ind].erase!
              ind = ind - @down_level
              #flag = 1
          end
          
        elsif sel[ind].faces.length == 3
          #flag = 0
          if    (sel[ind].faces[0].normal.angle_between(sel[ind].faces[1].normal) == 0)
            sel[ind].erase!
            ind = ind-@down_level
            #flag = 1
          elsif (sel[ind].faces[0].normal.angle_between(sel[ind].faces[2].normal) == 0)
            sel[ind].erase!
            ind = ind-@down_level
            #flag = 1
          elsif (sel[ind].faces[1].normal.angle_between(sel[ind].faces[2].normal) == 0)
            sel[ind].erase!
            ind = ind-@down_level
            #flag = 1
          elsif    (sel[ind].faces[0].normal.angle_between(sel[ind].faces[1].normal.reverse!) == 0)
            sel[ind].erase!
            ind = ind-@down_level
            #flag = 1
          elsif (sel[ind].faces[0].normal.angle_between(sel[ind].faces[2].normal.reverse!) == 0)
            sel[ind].erase!
            ind = ind-@down_level
            #flag = 1
          elsif (sel[ind].faces[1].normal.angle_between(sel[ind].faces[2].normal.reverse!) == 0)
            sel[ind].erase!
            ind = ind-@down_level
            #flag = 1
          end
          
        elsif sel[ind].faces.length == 4
          #flag = 0
          if    (sel[ind].faces[0].normal.angle_between(sel[ind].faces[1].normal) == 0)
            sel[ind].erase!
            ind = ind-@down_level
            #flag = 1
          elsif (sel[ind].faces[0].normal.angle_between(sel[ind].faces[2].normal) == 0)
            sel[ind].erase!
            ind = ind-@down_level
            #flag = 1
          elsif (sel[ind].faces[0].normal.angle_between(sel[ind].faces[3].normal) == 0)
            sel[ind].erase!
            ind = ind-@down_level
            #flag = 1
          elsif (sel[ind].faces[1].normal.angle_between(sel[ind].faces[2].normal) == 0)
            sel[ind].erase!
            ind = ind-@down_level
            #flag = 1
          elsif (sel[ind].faces[1].normal.angle_between(sel[ind].faces[3].normal) == 0)
            sel[ind].erase!
            ind = ind-@down_level
            #flag = 1
          elsif (sel[ind].faces[2].normal.angle_between(sel[ind].faces[3].normal) == 0)
            sel[ind].erase!
            ind = ind-@down_level
            #flag = 1
          elsif    (sel[ind].faces[0].normal.angle_between(sel[ind].faces[1].normal.reverse!) == 0)
            sel[ind].erase!
            ind = ind-@down_level
            #flag = 1
          elsif (sel[ind].faces[0].normal.angle_between(sel[ind].faces[2].normal.reverse!) == 0)
            sel[ind].erase!
            ind = ind-@down_level
            #flag = 1
          elsif (sel[ind].faces[0].normal.angle_between(sel[ind].faces[3].normal.reverse!) == 0)
            sel[ind].erase!
            ind = ind-@down_level
            #flag = 1
          elsif (sel[ind].faces[1].normal.angle_between(sel[ind].faces[2].normal.reverse!) == 0)
            sel[ind].erase!
            ind = ind-@down_level
            #flag = 1
          elsif (sel[ind].faces[1].normal.angle_between(sel[ind].faces[3].normal.reverse!) == 0)
            sel[ind].erase!
            ind = ind-@down_level
            #flag = 1
          elsif (sel[ind].faces[2].normal.angle_between(sel[ind].faces[3].normal.reverse!) == 0)
            sel[ind].erase!
            ind = ind-@down_level
            #flag = 1
          end
        end #main elsif 
      end #if sel[ind].class == Sketchup::Edge
      ind = ind + 1
    end #while
    end#2.times do
    model.commit_operation #Delete Coplanar Edges Loose
    
  end #def Copedges::delete_coplanar_edges_loose



  #***************************************************************************************************************
  #                                     ***Delete_Coplanar_Edges_Strict***
  #     This function is a variation of Delete_Coplanar_Edges_Loose (See above). It is basically the same thing,
  # except it does not check for opposite normals. Sometimes this works better and sometimes it doesnt. 
  #***************************************************************************************************************
  def Copedges::delete_coplanar_edges_strict
    
    model = Sketchup.active_model
    model.start_operation "Delete Coplanar Edges (strict)"
    
    sel = model.selection
    if sel.length < @down_level
      @down_level = sel.length
    end
    ind = 0
    while ind < sel.length
      if sel[ind].class == Sketchup::Edge
        
        if sel[ind].faces.length == 0
          sel[ind].erase!
          ind = ind - @down_level
          
        elsif sel[ind].faces.length == 2
          if (sel[ind].faces[0].normal.angle_between(sel[ind].faces[1].normal) == 0)
            sel[ind].erase!
            ind = ind - @down_level
          end
        
        elsif sel[ind].faces.length == 3
          if    (sel[ind].faces[0].normal.angle_between(sel[ind].faces[1].normal) == 0)
            sel[ind].erase!
            ind = ind-@down_level
          elsif (sel[ind].faces[0].normal.angle_between(sel[ind].faces[2].normal) == 0)
            sel[ind].erase!
            ind = ind-@down_level
          elsif (sel[ind].faces[1].normal.angle_between(sel[ind].faces[2].normal) == 0)
            sel[ind].erase!
            ind = ind-@down_level
          end
          
        elsif sel[ind].faces.length == 4
          if    (sel[ind].faces[0].normal.angle_between(sel[ind].faces[1].normal) == 0)
            sel[ind].erase!
            ind = ind-@down_level
          elsif (sel[ind].faces[0].normal.angle_between(sel[ind].faces[2].normal) == 0)
            sel[ind].erase!
            ind = ind-@down_level
          elsif (sel[ind].faces[0].normal.angle_between(sel[ind].faces[3].normal) == 0)
            sel[ind].erase!
            ind = ind-@down_level
          elsif (sel[ind].faces[1].normal.angle_between(sel[ind].faces[2].normal) == 0)
            sel[ind].erase!
            ind = ind-@down_level
          elsif (sel[ind].faces[1].normal.angle_between(sel[ind].faces[3].normal) == 0)
            sel[ind].erase!
            ind = ind-@down_level
          elsif (sel[ind].faces[2].normal.angle_between(sel[ind].faces[3].normal) == 0)
            sel[ind].erase!
            ind = ind-@down_level
          end
        end #if sel[ind].faces.length = number
      end #if sel[ind].class == Sketchup::Edge
      ind = ind + 1
    end #While loop
    model.commit_operation #Delete Coplanar Edges Strict
  end #Delete Coplanar Edges Strict
  
end  # Class Copedges

# -------------------------------------------------------------------------------------------------

if( not file_loaded?("deletecoplanaredges.rb"))
   UI.add_context_menu_handler do |menu|
      menu.add_separator if Copedges.validate_selection
                #The strict version of the code is useful for flat surfaces. The loose version is much better though.
      #menu.add_item("Delete Coplanar Edges (strict)") {Copedges.delete_coplanar_edges_strict} if Copedges.validate_selection
      menu.add_item("Delete Coplanar Edges") {Copedges.delete_coplanar_edges_loose} if Copedges.validate_selection
               #only add this if you are using prep faces for testing.
      #menu.add_item("Prep Faces") {Copedges.prep_faces} if Copedges.validate_selection
   end
end

file_loaded("deletecoplanaredges.rb")
